<?php

class Metadata_Manager {

	var $file;
	var $headerData = NULL;
	var $irbData = NULL;
	var $iptcData = NULL;
	var $xmpData = NULL;

	
	function __construct( $file )
	{
		$this->file = $file;
		$this->headerData = $this->get_header_data();
	}


	function set_iptc( $data )
	{
		// Beispiel: $data = array( '2:116' => 'Copyright by Mustermann', '2:25' => 'keyword1' )

		$iptc = $this->get_iptc_data();
		$count = count( $iptc );

		// alte Daten entfernen

		for ( $i = 0; $i < $count; $i++ )
		{
			if ( isset( $data[ $iptc[ $i ]['IPTC_Type'] ] ) )
			{
				unset( $iptc[ $i ] );
			}
		}

		$iptc = array_values( $iptc ); // Neuindizierung erzwingen

		// neue Daten hinzufügen

		foreach ( $data as $key => $value )
		{
			foreach ( (array) $value as $item )
			{
				$iptc[] = array(
					'IPTC_Type' => $key,
					'RecData' => $item 
				);
			}
		}

		$this->put_iptc_data( $iptc );
		return $iptc;
	}


	function write( $outputfile )
	{
		$imageData = $this->get_image_data( $this->file );

		if ( $imageData === FALSE || $imageData === NULL )
		{
			return FALSE;
		}

		foreach ( $this->headerData as $segment )
		{
			if ( strlen( $segment['SegData'] ) > 0xfffd )
			{
				return FALSE; // Header ist größer als erlaubt
			}
		}

		ignore_user_abort( TRUE );
		
		if ( gettype( $outputfile ) == 'resource' )
		{
			$handle = $outputfile;
		}
		else
		{
			$handle = fopen( $outputfile, 'wb' );
		}

		if ( ! $handle  )
		{
			return FALSE;
		}

		fwrite( $handle, "\xFF\xD8" ); // SOI schreiben

		foreach ( $this->headerData as $segment )
		{
			fwrite( $handle, sprintf( "\xFF%c", $segment['SegType'] ) ); // Segmentmarker schreiben
			fwrite( $handle, pack( 'n', strlen( $segment['SegData'] ) + 2 ) ); // Segmentgröße schreiben
			fwrite( $handle, $segment['SegData'] ); // Segmentdaten schreiben
		}

		fwrite( $handle, $imageData ); // Bilddaten schreiben
		fwrite( $handle, "\xFF\xD9" ); // EOI schreiben

		if ( gettype( $outputfile ) == 'string' )
		{
			fclose( $handle );
		}

		ignore_user_abort( FALSE );
		return TRUE;
	}


	function deliver( $filename = '' )
	{
		$handle = tmpfile();

		if ( ! $handle  )
		{
			die( 'Could not open temporary file for writing' );
		}

		if ( ! $this->write( $handle ) )
		{
			die( 'Could not create image file' );
		}

		rewind( $handle );
		$stat = fstat( $handle );
		header( 'Content-Description: File Transfer' );
		header( 'Content-Type: image/jpeg' );
		header( 'Content-Transfer-Encoding: binary' );
		header( 'Expires: 0' );
		header( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
		header( 'Pragma: public' );
		header( 'Content-Length: ' . $stat['size'] );

		if ( empty( $filename ) )
		{
			header( 'Content-Disposition: inline; filename="' . basename( $this->file ) . '"' );
		}
		else
		{
			header( 'Content-Disposition: attachment; filename="' . $filename . '"' );
		}

		ob_end_clean();

		while ( ! feof( $handle ) )
		{
			print( fread( $handle, 1024*8 ) );
			flush();
			if ( connection_status() != 0 )
			{
				fclose( $handle );
				exit;
			}
		}

		fclose( $handle );
		exit;
	}


	function get_iptc_data()
	{
		if ( $this->iptcData !== NULL )
		{
			return $this->iptcData;
		}

		$irb = $this->get_irb_data();
		$count = count( $irb );

		for( $i = 0; $i < $count; $i++ )
		{
			if ( $irb[ $i ]['ResID'] == 0x0404 )
			{
				$data = $irb[ $i ]['ResData'];
				break;
			}
		}

		if ( empty( $data ) )
		{
			return FALSE;
		}

		$pos = 0;
		$length = strlen( $data );
		$this->iptcData = array( );

		while( $pos < $length )
		{
			if ( strlen( substr( $data, $pos ) ) < 5 )
			{
				// Nicht genügend Daten übrig für einen Datensatz - Daten vermutlich korrupt
				// Aber liefere zumindest Teildaten zurück
				return $this->iptcData;
			}

			// IPTC-Datensatz entpacken:
			// Byte 1: IPTC-Tag-Marker (immer 28)
			// Byte 2: Nummer des IPTC-Records
			// Byte 3: Nummer des IPTC-Datasets
			// Bytes 4 und 5: Länge
			$iptcRaw = unpack( 'CIPTC_Tag_Marker/CIPTC_Record_No/CIPTC_Dataset_No/nIPTC_Size', substr( $data, $pos ) );
			$pos += 5;

			// Stelle IPTC-Type-String zusammen
			$iptctype = sprintf( '%01d:%02d', $iptcRaw['IPTC_Record_No'], $iptcRaw['IPTC_Dataset_No'] );

			// Prüfen, ob genügend Daten vorhanden sind, um Datensatzinhalt zu lesen
			if ( strlen( substr( $data, $pos, $iptcRaw['IPTC_Size'] ) ) !== $iptcRaw['IPTC_Size'] )
			{
				// Nicht genügend Daten übrig für Datensatzinhalt - Daten vermutlich korrupt
				// Aber liefere zumindest Teildaten zurück
				return $this->iptcData;
			}

			// IPTC-Datensatz zum Ergebnis hinzufügen
			$this->iptcData[] = array( 
				'IPTC_Type' => $iptctype ,
				'RecData' => substr( $data, $pos, $iptcRaw['IPTC_Size'] ) 
			);

			$pos += $iptcRaw['IPTC_Size'];
		}

		return $this->iptcData;
	}


	function put_iptc_data( $newIptcBlock )
	{
		$this->iptcData = $newIptcBlock;
		$irbData = $this->get_irb_data();
		$iptcBlockPos = count( $irbData );

		for( $i = 0; $i < $iptcBlockPos; $i++ )
		{
			if ( $irbData[ $i ]['ResID'] == 0x0404 )
			{
				$iptcBlockPos = $i;
			}
		}

		$iptcData = '';

		foreach ( $newIptcBlock as $record )
		{
			$type = sscanf( $record['IPTC_Type'], '%d:%d' );

			// Schreibe IIM-Tag-Marker, Record-Nummer, Dataset-Nummer und Datenlänge in die gepackten Ausgabe
			$iptcData .= pack( 'CCCn', 28, $type[0], $type[1], strlen( $record['RecData'] ) );

			// Schreibe IIM-Daten in die gepackten Ausgabe
			$iptcData .= $record['RecData'];
		}

		$irbData[ $iptcBlockPos ] = array(
			'ResID' => 0x0404,
			'ResEmbeddedName' => "\x00\x00",
			'ResData' => $iptcData 
		);

		$this->put_irb_data( $irbData );
	}
	
	
	function get_xmp_data()
	{
		if ( $this->xmpData !== NULL )
		{
			return $this->xmpData;
		}

		$count = count( $this->headerData );

		for ( $i = 0; $i < $count; $i++ )
		{
			if ( $this->headerData[ $i ]['SegType'] == 0xE1 )
			{
				if ( substr( $this->headerData[ $i ]['SegData'], 0, 29 ) == "http://ns.adobe.com/xap/1.0/\x00" )
				{
					$this->xmpData = substr( $this->headerData[ $i ]['SegData'], 29 );
					return $this->xmpData;
				}
			}
		}

		$this->xmpData = '';
		return '';
	}
	
	
	function put_xmp_data( $xmpString )
	{
		for ( $i = 0; $i < count( $this->headerData ); $i++ )
		{
			if ( $this->headerData[ $i ]['SegType'] == 0xE1 
					&& substr( $this->headerData[ $i ]['SegData'], 0, 29 ) == "http://ns.adobe.com/xap/1.0/\x00" )
			{
				// Vorhandenen XMP/RDF-Block gefunden - ersetze ihn mit dem neuen
				$this->headerData[ $i ]['SegData'] = "http://ns.adobe.com/xap/1.0/\x00" . $xmpString;
				return;
			}
		}

		$i = 0;

		// Durchlaufe Header, bis ein Block gefunden wird, der nicht APP0 or APP1 ist
		while ( $this->headerData[ $i ]['SegType'] == 0xE0 || $this->headerData[$i]['SegType'] == 0xE1 )
		{
			$i++;
		}

		// Füge ein neues XMP/RDF-APP1-Segment an der angegeben Position ein
		array_splice( $this->headerData, $i, 0, array( array(
			'SegType' => 0xE1,
			'SegData' => "http://ns.adobe.com/xap/1.0/\x00" . $xmpString ) ) 
		);
	}


	function get_irb_data()
	{
		if ( $this->irbData !== NULL )
		{
			return $this->irbData;
		}

		// Photoshop-Image-Resource-Blöcke können mehrere JPEG-APP13-Segmente umfassen,
		// darum müssen wir sie verbinden, falls es mehrere gibt

		$irb = '';
		$count = count( $this->headerData );

		for ( $i = 0; $i < $count; $i++ )
		{
			if ( $this->headerData[ $i ]['SegType'] == 0xED 
					&& substr( $this->headerData[ $i ]['SegData'], 0, 14) == "Photoshop 3.0\x00" )
			{
				$irb .= substr ( $this->headerData[ $i ]['SegData'], 14 );
			}
		}

		if ( $irb == '' )
		{
			$this->irbData = array();
			return FALSE;
		}

		$length = strlen( $irb );
		$pos = strpos( $irb, '8BIM' );

		while ( $pos !== FALSE )
		{
			// "8BIM"-Zeichen überspringen
			$pos += 4;

			// Die nächsten beiden Zeichen bilden die Datensatz-ID und geben die Art des Datensatzes an
			$id = ord( $irb[ $pos ] ) * 256 + ord( $irb[ $pos + 1 ] );
			$pos += 2;

			// Als nächstes kommt der Datensatzname, der meistens leer ist
			// (eine Zeichenkette, die mit NULL endet und mit 0x00-Werten auf eine gerade Länge gebracht ist)
			$namePos = $pos;

			// HINWEIS: Photoshop speichert die Namen von Ressourcen nicht standardkonform
			// (gemäß Photoshop-6.0-Dateiformatspezifikation von November 2000)
			//
			// Der Ressourcenname ist folgendermaßen aufgebaut:
			// Länge des Namens (1 Byte), eigentlicher Name, NULL-Byte als Abschluss,
			// gegebenenfalls weitere NULL-Bytes, um eine gerade Länge des Ressourcennamens sicherzustellen

			// Länge ermitteln
			$nameLength = ord ( $irb[ $namePos ] );

			// Längenangabe und Name müssen zusammen eine gerade Zahl von Bytes bilden,
			// deshalb muss der Name eine ungerade Länge haben
			if ( $nameLength % 2 == 0 )
			{
				// gegebenfalls ein Byte hinzufügen
				$nameLength ++;
			}

			$pos += $nameLength + 1;

			// Als nächstes kommt die Längenangabe der eigentlichen Daten (4 Bytes, MSB first)
			$datasize = ord( $irb[ $pos ] ) * 16777216 + ord( $irb[ $pos + 1 ] ) * 65536 + ord( $irb[ $pos + 2 ] ) * 256 + ord( $irb[ $pos + 3 ] );
			$pos += 4;

			// Lies die Beschreibung für dieses Ressource
			// Prüfe, ob es sich um eine "Path-Information"-Ressource handelt (die einen Bereich von IDs hat)
			if ( ( $id >= 0x07D0 ) && ( $id <= 0x0BB6 ) )
			{
				$description = "ID Info : Path Information (saved paths).";
			}
			else
			{
				$description = isset( $this->photoshop_irb_descriptions[ $id ] ) ? $this->photoshop_irb_descriptions[ $id ] : '';
			}

			$this->irbData[] = array(
				"ResID" => $id,
				"ResEmbeddedName" => trim( substr ( $irb, $namePos + 1,  $nameLength ) ),
				"ResData" => substr( $irb, $pos, $datasize ) 
			);

			// Berücksichtige, dass der Datensatz u. U. mit NULL-Bytes auf eine gerade Länge gebracht wurde
			$pos += $datasize + ( $datasize % 2 );
			
			if ( $pos >= $length )
			{
				break;
			}

			$pos = strpos( $irb, '8BIM', $pos );
		}

		return $this->irbData;
	}
	
	
	function put_irb_data( $newData )
	{
		// Lösche alle vorhandenen Photoshop-IRB-Blöcke

		$count = count( $this->headerData );

		for ( $i = 0; $i < $count ; $i++ )
		{
			if ( $this->headerData[ $i ]['SegType'] == 0xED
					&& substr( $this->headerData[ $i ]['SegData'], 0, 14 ) == "Photoshop 3.0\x00" )
			{
				unset( $this->headerData[ $i ] );
			}
		}

		// Hole gepackte Photoshop-IRB-Daten

		$packedData = $this->_pack_irb_data( $newData );

		// Durchlaufe Header-Segmente rückwärts, um Einfügestelle zu finden (nach einem APP0- bis APP12-Block)

		$i = count( $this->headerData ) - 1;

		while ( $i >= 0 && ( $this->headerData[ $i ]['SegType'] > 0xED || $this->headerData[ $i ]['SegType'] < 0xE0 ) )
		{
			$i--;
		}

		// Verteile gepackte Ausgabedaten auf 32.000 Byte große APP13-Segmente

		while ( strlen( $packedData ) > 32000 )
		{
			array_splice( $this->headerData, $i + 1, 0, array(
				'SegType' => 0xED,
				'SegData' => "Photoshop 3.0\x00" . substr( $packedData, 0, 32000 ) ) 
			);

			$packedData = substr_replace( $packedData, '', 0, 32000 );
			$i++;
		}

		// Schreibe Restdaten in ein APP13-Segment
		array_splice( $this->headerData, $i + 1 , 0, '' );

		$this->headerData[ $i + 1 ] =  array(
			'SegType' => 0xED,
			'SegData' => "Photoshop 3.0\x00" . $packedData 
		);
	}
	
	
	function _pack_irb_data( $data )
	{
		$result = '';

		foreach ( $data as $resource )
		{
			if ( strlen( $resource['ResData'] ) == 0 )
			{
				continue;
			}

			$result .= pack( 'a4n', '8BIM', $resource['ResID'] );

			// Namenslänge
			$result .= pack( 'c', strlen( trim( $resource['ResEmbeddedName'] ) ) );

			// Name
			$result .= trim( $resource['ResEmbeddedName'] );

			// ggf. mit NULL-Bytes auffüllen, um gerade Anzahl an Bytes zu erreichen
			if ( strlen( trim( $resource['ResEmbeddedName'])  ) % 2 == 0 )
			{
				$result .= "\x00";
			}

			// Datenlänge
			$result .= pack( 'N', strlen( $resource['ResData'] ) );

			// Daten
			$result .= $resource['ResData'];

			// ggf. mit NULL-Bytes auffüllen, um gerade Anzahl an Bytes zu erreichen
			if ( strlen( $resource['ResData'] ) % 2 == 1 )
			{
				$result .= "\x00";
			}
		}

		return $result;
	}

	
	function get_header_data()
	{
		$handle = fopen( $this->file, 'rb' );

		if ( ! $handle  )
		{
			return FALSE;
		}

		if ( fread( $handle, 2 ) != "\xFF\xD8" )
		{
			fclose( $handle );
			return FALSE; // kein SOI am Anfang, d. h. vermutlich keine JPEG-Datei
		}

		$result = array();
		$data = fread( $handle, 2 );

		while ( $data[0] == "\xFF" && $data[1] != "\xD9" && ! feof( $handle ) )
		{
			// wenn der Segment-Marker kein "Restart"-Marker (ohne nachfolgende Längenangabe und Daten) ist
			if (  ( ord( $data[1] ) < 0xD0 ) || ( ord( $data[1] ) > 0xD7 ) )
			{
				// Datenlänge lesen und in Ganzzahl umwandeln
				$sizestr = fread( $handle, 2 );
				$decodedsize = unpack( 'nsize', $sizestr );

				// Startposition der Daten merken
				$segdatastart = ftell( $handle );

				// Segmentddaten lesen und zum Ergebnis hinzufügen
				$segdata = fread( $handle, $decodedsize['size'] - 2 );
				$result[] = array(
					'SegType' => ord( $data[1] ),
					'SegDataStart' => $segdatastart,
					'SegData' => $segdata
				);
			}

			if ( $data[1] == "\xDA" )
			{
				// SOS-Segment (start of scan), daher Ende der Header-Daten
				break;
			}

			$data = fread( $handle, 2 );
		}

		fclose( $handle );
		return $result;
	}


	function get_image_data()
	{
		$handle = fopen( $this->file, 'rb' );

		if ( ! $handle  )
		{
			return FALSE;
		}

		$data = fread( $handle, 2 );

		if ( $data != "\xFF\xD8" )
		{
			fclose( $handle );
			return FALSE; // kein SOI am Dateianfang
		}

		$data = fread( $handle, 2 );

		while ( $data[0] == "\xFF" && $data[1] != "\xD9" && ! feof( $handle ) )
		{
			// wenn der Segment-Marker kein "Restart"-Marker (ohne nachfolgende Längenangabe und Daten) ist
			if ( ord( $data[1] ) < 0xD0 || ord( $data[1] ) > 0xD7 )
			{
				// Datenlänge lesen und in Ganzzahl umwandeln
				$sizestr = fread( $handle, 2 );
				$decodedsize = unpack( 'nsize', $sizestr );

				// Segmentdaten lesen
				$segdata = fread( $handle, $decodedsize['size'] - 2 );
			}

			if ( $data[1] == "\xDA" )
			{
				// SOS-Segment (start of scan), daher Ende der Header-Daten / Beginn der komprimierten Bilddaten

				$compressedData = fread( $handle, filesize( $this->file ) );
				$compressedData = substr( $compressedData, 0, strpos( $compressedData, "\xFF\xD9" ) ); // EOI und alles Nachfolgende abschneiden
				fclose( $handle );
				return $compressedData;
			}

			$data = fread( $handle, 2 );
		}

		fclose( $handle );
		return FALSE;
	}
}


// end of file