<?php

// Klasse fr Picturemaxx-Anbindungen vom Typ XML3

class PmConnector {
	
	var $db; // mysqli object for database operations
	var $languages; // array holding an ordered list of content languages to be considered for the current connection
	var $userId; // id of the PTM user
	var $userGroups; // array holding the ids of groups the user belongs to
	var $pageIdImages; // internal id of the images root page
	var $metadata; // array holding a conversion table for metadata tags
	var $globals; // array holding Pixtacy's global variables
	var $request; // array holding all the parameters of the original request
	var $error; // variable holding the last error message
	var $sessionDuration	= 3600;

	var $mysqlServer;
	var $mysqlDatabase;
	var $mysqlUser;
	var $mysqlPassword;
	var $mysqlTablePrefix;
	
	var $cfg; // array holding the configuration parameters of the Pixtacy environment

	var $tblContent;
	var $tblImages;
	var $tblImagelists;
	var $tblUsers;
	var $tblIndex;
	var $tblRoot;
	
	
	function __construct( $pmConfig = array() )
	{
		$path = realpath( dirname( __FILE__ ) . '/../../../config/config.php' );
		date_default_timezone_set('Europe/Berlin');

		if ( $path && $cfg = $this->read_config( $path ) )
		{
			$this->cfg = $cfg;
			
			$this->mysqlServer = $cfg['mysqlServer'];
			$this->mysqlDatabase = $cfg['mysqlDatabase'];
			$this->mysqlUser = $cfg['mysqlUser'];
			$this->mysqlPassword = $cfg['mysqlPassword'];
			$this->mysqlTablePrefix = $cfg['mysqlTablePrefix'];

			$this->tblContent = $this->mysqlTablePrefix . 'content';
			$this->tblImages = $this->mysqlTablePrefix . 'data_image';
			$this->tblImagelists = $this->mysqlTablePrefix . 'data_imagelist';
			$this->tblUsers = $this->mysqlTablePrefix . 'user';
			$this->tblIndex = $this->mysqlTablePrefix . 'index';
			$this->tblRoot = $this->mysqlTablePrefix . 'data_vtroot';
			
			$this->metadata['keywords'] = 'keywords';
			$this->metadata['date_created'] = 'creationdate';
			$this->metadata['by_line'] = 'photographer';
			$this->metadata['headline'] = 'name';
			$this->metadata['credit'] = 'credit';
			$this->metadata['copyright_notice'] = 'copyright';
			$this->metadata['caption'] = 'remarks';
			$this->metadata['author_code'] = 'photographer';
			$this->metadata['city'] = 'city';
			$this->metadata['country_primary_location_name'] = 'country';
			$this->metadata['country_primary_location_code'] = 'countrycode';
			$this->metadata['province_state'] = 'state';
			$this->metadata['special_instructions'] = 'instructions';
		}
	}
	
	
	function connect( $request )
	{
		if ( is_object( $this->db ) && get_class( $this->db ) == 'mysqli' )
		{
			return TRUE;
		}

		if ( ! $this->mysqlServer )
		{
			$this->error('configuration data missing');
			return FALSE;
		}

		$this->db = new mysqli( $this->mysqlServer, $this->mysqlUser, $this->mysqlPassword, $this->mysqlDatabase );

		if ( $this->db->connect_error )
		{
			$this->error('could not connect to mysql server');
			return FALSE;
		}

		if ( ! $this->pageIdImages = $this->get_images_page_id() )
		{
			$this->error('could not find root images page');
			return FALSE;
		}

		$this->db->query("SET NAMES 'latin1'");
		$this->request = $request;
		return TRUE;
	}
	
	
	function disconnect()
	{
		// kept for compatibility reasons
	}

	
	function login( $username, $password )
	{
		$this->userId = $this->get_user_id( $username, $password );

		if ( ! $this->userId )
		{
			$this->error('login failed');
			return FALSE;
		}
		
		if ( session_status() == PHP_SESSION_NONE )
		{
			session_start( array ( 'use_cookies' => 0 ) );
		}

		$_SESSION['pm_user_id'] = $this->userId;
		$_SESSION['pm_expiration_time'] = time() + $this->sessionDuration;
		return array ( 'id' => session_id(), 'duration' => $this->sessionDuration );
	}

	
	function check_session()
	{
		if ( empty( $this->request['session'] ) )
		{
			return FALSE;
		}

		session_id( $this->request['session'] );
		session_start( array ( 'use_cookies' => 0 ) );

		if ( empty( $_SESSION['pm_expiration_time'] ) 
				|| $_SESSION['pm_expiration_time'] < time() )
		{
			return FALSE;
		}
		
		$this->userId = $_SESSION['pm_user_id'];
		$_SESSION['pm_expiration_time'] = time() + $this->sessionDuration;
		return TRUE;
	}

	
	function count( $search = array(), $prefixSearch = FALSE )
	{
		$conditions = $this->build_where_clause( $search, $prefixSearch );
		$sql = "SELECT COUNT(*) AS count 
						FROM `{$this->tblContent}` c, `{$this->tblImages}` d, `{$this->tblIndex}` i 
						WHERE $conditions";
		$result = $this->db->query( $sql );

		if ( ! $result || ! $result->num_rows )
		{
			$this->error( $this->db->error );
			return -1;
		}
		
		$row = $result->fetch_assoc();
		$result->free();
		return $row['count'];
	}
	
	
	function search( $search, $maxresults = 0, $sortfield = 'datearchivation', $sortorder = 'desc', $prefixSearch = FALSE )
	{
		// $search = array( $criteria1, $criteria2, ... )
		// $criteria = $comparison or $or_block
		// $or_block = array( $comparison, $comparison, ... )
		// $comparison = array( $field, $operator, $value )
		// $operator = contains|equal|greater|greater-or-equal|less|less-or-equal|
		//             not-contains|not-equal|not-greater etc.

		$conditions = $this->build_where_clause( $search, $prefixSearch );
		$sorting = $this->build_order_clause( $sortfield, $sortorder );

		$sql = "SELECT LEFT( d.`dateiname`, LENGTH( d.`dateiname` ) - 4 ) as id, "
					."DATE_FORMAT( FROM_UNIXTIME( c.`creationTime` ), '%Y%m%d' ) AS archiveDate, "
					."d.`creationdate` AS exposureDate, "
					."'' AS height, '' AS width "
					."FROM `{$this->tblContent}` c, `{$this->tblImages}` d, `{$this->tblIndex}` i "
					."WHERE $conditions ORDER BY $sorting";

		if ( $maxresults > 0 )
		{
			$sql .= " LIMIT 0,$maxresults";
		}
		
		$result = $this->db->query( $sql );
		
		if ( ! $result )
		{
			return -1;
		}

		$images = array();
		
		while ( $row = $result->fetch_assoc() )
		{
			$images[] = $row;
		}

		if ( $result->num_rows == 0 )
		{
			$count = 0;
		}
		elseif ( $maxresults == 0 || $result->num_rows < $maxresults )
		{
			$count = -1;
		}
		else
		{
			// get total number of found images
			$result->free();
			$sql = "SELECT COUNT(*) AS count 
							FROM `{$this->tblContent}` c, `{$this->tblImages}` d, `{$this->tblIndex}` i 
							WHERE $conditions";
			$result = $this->db->query( $sql );

			if ( ! $result )
			{
				return -1;
			}

			$row = $result->fetch_assoc();
			$result->free();
			$count = $row['count'] == $maxresults ? -1 : $row['count'];
		}

		return array( 'count' => $count, 'assets' => $images );
	}
	
	
	function asset_metadata( $ref, $tags = array() )
	{
		// returns metadata for a specific asset
		// return array( array( 'field' => 'keywords', 'value' => 'woman' ), array( 'field' => 'date_created', 'value' => '20100923' ) )

		if ( empty( $tags ) )
		{
			$tags = $this->metadata_list();
		}
		
		$columns = $this->tag2column( $tags );
		$fieldlist = '`' . implode( '`,`', $columns ) . '`';
		$sql = "SELECT $fieldlist FROM `{$this->tblImages}` WHERE `dateiname` LIKE '$ref%'";
		$result = $this->db->query( $sql );
		
		if ( ! $result || ! $result->num_rows )
		{
			return $result ? -2 : -1;
		}
		
		$row = $result->fetch_assoc();
		$result->free();
		$result = array();
		
		foreach ( $columns as $column )
		{
			if ( $column == 'keywords')
			{
				foreach ( explode( ',', $row['keywords'] ) as $keyword )
				{
					$result[] = array( 'field' => 'keywords', 'value' => html_entity_decode( trim( $keyword ) ) );
				}
			}
			else
			{
				$result[] = array( 'field' => $this->column2tag( $column ), 'value' => strip_tags( html_entity_decode( $row[ $column ] ) ) );
			}
		}

		return $result;
	}
	
	
	function asset_permissions( $ref, $clearing = FALSE )
	{
		// returns permissions and clearing information for a specific asset

		foreach ( array( 'view', 'download' ) as $type )
		{
			foreach ( array( 'thumb', 'preview', 'highres' ) as $resolution )
			{
				$permissions[ $type ][ $resolution ] = $this->get_permission( $ref, $resolution, $type ) ? '1' : '0';
			}
		}
		
		if ( $permissions['download']['highres'] == '1' )
		{
			$permissions['download']['highres'] = '0';
			$sql = "SELECT `verzeichnis`, `dateiname` FROM `{$this->tblImages}` WHERE `dateiname` LIKE '$ref%'";
			$result = $this->db->query( $sql );
		
			if ( $result && $result->num_rows )
			{
				$row = $result->fetch_assoc();
				$result->free();
				$this->load_globals();
				$dir = $this->globals['pxtcPathHires'];
			
				if ( $dir[0] != '/' )
				{
					$dir = dirname( __FILE__ ) . '/../../../' . $dir;
				}
			
				if ( realpath( "$dir/{$row['verzeichnis']}/{$row['dateiname']}" ) )
				{
					$permissions['download']['highres'] = '1';
				}
			}
		}

		return $permissions;
	}
	
	
	function download( $ref, $resolution, $download = TRUE, $comment = '' )
	{
		// returns a file reference of a specific asset
		
		$sql = "SELECT `verzeichnis`, `dateiname` FROM `{$this->tblImages}` WHERE `dateiname` LIKE '$ref%'";
		$result = $this->db->query( $sql );
		
		if ( ! $result || ! $result->num_rows )
		{
			return $result ? -2 : -1;
		}
		
		$row = $result->fetch_assoc();
		$result->free();
		$resArray = array( '1' => 'thumb', '2' => 'preview', '3' => 'highres' );
		
		if ( ! in_array( $resolution, $resArray ) )
		{
			if ( ! isset( $resArray[ $resolution ] ) )
			{
				return -3;
			}
			
			$resolution = $resArray[ $resolution ];
		}
		
		$type = $download ? 'download' : 'view';
		
		if ( ! $this->get_permission( $ref, $resolution, $type ) )
		{
			return -4;
		}

		if ( ! $this->load_globals() )
		{
			return -1;
		}

		switch ( $resolution )
		{
			case 'thumb':
				$dir = "data/pixtacy_thumbnails";
				$notify = 0;
				break;
			case 'preview':
				$dir = $this->globals['pxtcPathLores'];
				$notify = $this->globals['pxtcNotifyLoresDownload'];
				break;
			case 'highres':
				$dir = $this->globals['pxtcPathHires'];
				$notify = $this->globals['pxtcNotifyHiresDownload'];
		}
		
		if ( $dir[0] != '/' )
		{
			$dir = dirname( __FILE__ ) . '/../../../' . $dir;
		}
		
		$file = realpath( "$dir/{$row['verzeichnis']}/{$row['dateiname']}" );
		
		if ( ! $file )
		{
			return -5;
		}
		
		if ( ! empty( $this->globals['mailRecipient'] ) && $notify && $download )
		{
			$subject = 'Pixtacy-Download ber Picturemaxx';
			$message = 'ber Picturemaxx wurde am ' . date('d.m.Y') . ' um ' . date('H:i') 
						. " Uhr die folgende Bilddatei heruntergeladen:\n"
						. "\n"
						. "$file\n"
						. "\n"
						. "Kundennummer: {$this->request['client_id']}\n"
						. "Kundengruppe(n): {$this->request['client_group_name']}\n"
						. "Land/Sprache: {$this->request['client_country_name_iso3']}/{$this->request['client_language_name_iso3']}\n"
						;
			
			if ( $comment )
			{
				$message .= "\nBemerkungen:\n$comment\n";
			}
			
			mail( $this->globals['mailRecipient'], $subject, $message );
		}

		return $file;
	}
	
	
	function download_app( $assets = array() )
	{
		// returns url to a web page where user can increase credit limit
	}
	
	
	function author_list()
	{
		// returns a list of authors (photographers or agencies)

		$sql = "SELECT d.`photographer` 
						FROM `{$this->tblContent}` c, `{$this->tblImages}` d, `{$this->tblIndex}` i 
						GROUP BY d.`photographer`";
		$result = $this->db->query( $sql );
		
		if ( ! $result )
		{
			return -1;
		}
		
		$authorlist = array();
		
		while ( $row = $result->fetch_assoc() )
		{
			if ( ! empty( $row['photographer'] ) )
			{
				$authorlist[] = array( 'ref' => $row['photographer'], 'name' => $row['photographer'] );
			}
		}

		$result->free();
		return $authorlist;
	}
	
	
	function offer_list( $datefrom, $dateto, $maxresults = 0 )
	{
		// returns a list of offers
		return array();
	}
	
	
	function offer_assets( $ref, $maxresults = 0 )
	{
		// returns a list of assets for a specific offer
		return array();
	}
	
	
	function metadata_list()
	{
		// returns a list of supported metadata fields
	
		return array_keys( $this->metadata );
	}
	

	#########################
	#### Private Methods ####
	#########################
	
	function error( $msg )
	{
		$this->error = $msg;

		if ( isset( $this->cfg['log'] ) && $this->cfg['log'] == '1' )
		{
			echo $msg;
		}
	}

	
	function get_user_id( $username, $password )
	{
		// checks if $username and $password are correct and returns id of the user

		$hash = md5( $password );
		$sql = "SELECT `id` FROM `{$this->tblUsers}` 
						WHERE `name` = '$username' AND `active`= 1 AND `password` = '$hash'";
		$result = $this->db->query( $sql );

		if ( ! $result || ! $result->num_rows  )
		{
			return FALSE;
		}

		$row = $result->fetch_assoc();
		$result->free();
		$sql = "UPDATE `{$this->tblUsers}` 
					SET `lastaccesstime` = UNIX_TIMESTAMP() 
					WHERE `vtid` = {$row['id']}";
		$this->db->query( $sql );
		return $row['id'];
	}

	
	function get_images_page_id()
	{
		$sql = "SELECT c2.`id` AS id FROM `{$this->tblContent}` AS c1 
						INNER JOIN `{$this->tblContent}` AS c2 ON c1.`id` = c2.`containerID` 
						WHERE c1.`package` = 'pixtacy' AND c1.`type` = 'index' 
						AND c2.`package` = 'pixtacy' AND c2.`type` = 'imagelist'";
		$result = $this->db->query( $sql );

		if ( ! $result || ! $result->num_rows )
		{
			return FALSE;
		}

		$row = $result->fetch_assoc();
		$result->free();
		return $row['id'];
	}


	function get_user_groups( $type = 'id' )
	{
		if ( ! empty( $this->userGroups ) )
		{
			return $this->userGroups[$type];
		}
		
		if ( $type != 'name' )
		{
			$type = 'id';
		}
		
		$sql = "SELECT c1.`id`, c1.`name` 
						FROM `{$this->tblContent}` AS c1 
						INNER JOIN `{$this->tblContent}` AS c2 ON c1.`id` = c2.`containerID` 
						WHERE c2.`aliasID` = {$this->userId}";
		$result = $this->db->query( $sql );
		
		if ( ! $result )
		{
			return FALSE;
		}

		$groupIds = array();
		$groupNames = array();

		while ( $row = $result->fetch_assoc() )
		{
			$groupIds[] = $row['id'];
			$groupNames[] = $row['name'];
		}
		
		$result->free();
		$this->userGroups['id'] = $groupIds;
		$this->userGroups['name'] = $groupNames;
		return $this->userGroups[$type];
	}


	function build_where_clause( $search = array(), $prefixSearch = FALSE )
	{
		$userIds = implode( ',', array_merge( array( $this->userId ), $this->get_user_groups() ) );
		$conditions = " c.`id` = d.`vtid` AND c.`id` = i.`id` AND c.`package` = 'pixtacy' AND c.`type` = 'image' "
						."AND FIND_IN_SET( {$this->pageIdImages}, c.`path` ) "
						."AND ( ( c.`ownerID` IN ($userIds) OR ( c.`userID` IN ($userIds) "
						."AND LEFT( RIGHT( c.`accessRights`, 2 ), 1 ) > 3 ) ) "
						."OR RIGHT( c.accessRights, 1 ) > 3 ) AND c.`userID` != 52";

		foreach ( $search as $criteria )
		{
			if ( isset( $criteria['field'] ) )
			{
				// single criteria
				$str = $this->build_condition( $criteria, $prefixSearch );
			
				if ( $str !== FALSE )
				{
					$conditions .= ' AND ' . $str;
				}
			}
			else
			{
				// OR Block
				$conditions .= ' AND ( ';
				$counter = 1;
				
				foreach ( $criteria as $subcriteria )
				{
					$str = $this->build_condition( $subcriteria, $prefixSearch );
				
					if ( $str !== FALSE )
					{
						$conditions .= ( $counter > 1 ? ' OR ' : '' ) . $str;
						$counter++;
					}
				}
				
				$conditions .= ' ) ';
			}
		}

		return $conditions;
	}
	

	function build_order_clause( $sortfield, $sortorder )
	{
		switch ( $sortfield )
		{
			case 'datearchivation':
				$str = "c.`creationTime`";
				break;
			case 'reference':
				$str = "d.`dateiname`";
				break;
			default:
				$str = "d.creationDate";
		}
		
		$str .= $sortorder == 'asc' ? ' ASC' : ' DESC';
		return $str;
	}

	
	function build_condition( $criteria, $prefixSearch = FALSE )
	{
		// erzeugt einen String, der als Bedingung in einer MySQL-Where-Klausel benutzt werden kann
		// $criteria muss ein assoziatives Array mit den Elementen 'field', 'operator', 'value' und optional 'metadataTag' sein

		extract( $criteria );
		$value = utf8_decode( $value );
		$str = '';

		switch ( $operator )
		{
			case 'not-contains':
				$str = ' NOT';
			case 'contains':
				$value = addcslashes( strtolower($value), '.\/+[](){}' );
				$value = str_replace( '?', '.', $value );
				$value = str_replace( '*', '.+', $value );
				$value = str_replace( array( 'ae', '' ), '(ae|)', $value );
				$value = str_replace( array( 'oe', '' ), '(oe|)', $value );
				$value = str_replace( array( 'ue', '' ), '(ue|)', $value );
				$value = str_replace( array( 'ss', '' ), '(ss|)', $value );
				$str .= " REGEXP '[[:<:]]{$value}";
				$str .= $prefixSearch ? "' " : "[[:>:]]' ";
				break;
			case 'equal':
				$str = " = '$value' ";
				break;
			case 'not-equal':
				$str = " != '$value' ";
				break;
			case 'greater':
			case 'not-less-or-equal':
				$str = " > '$value' ";
				break;
			case 'greater-or-equal':
			case 'not-less':
				$str = " >= $value ";
				break;
			case 'less':
			case 'not-greater-or-equal':
				$str = " < $value ";
				break;
			case 'less-or-equal':
			case 'not-greater':
				$str = " <= $value ";
				break;
		}

		if ( empty( $str ) )
		{
			return FALSE;
		}

		switch ( $field )
		{
			case 'author':
				$result = 'd.`photographer` ' . $str;
				break;
			case 'datecreated':
				$result = 'd.`creationdate` ' . $str;
				break;
			case 'optical':
				$result = '1'; // Pixtacy-Bilder liegen immer in digitaler Form vor
				break;
			case 'fulltext':
				$result = 'i.`index` ' . $str;
				break;
			case 'metadata':
				$column = $this->tag2column( $metadataTag );
				$result = $column ? "d.`{$column}` {$str}" : NULL;
				break;
			case 'mimegroup':
				$result = $value == 'image' ? '1' : '1 = 0'; // TODO: Bilder und Videos differenzieren
				break;
			case 'format':
				$result = 'd.`format` ' . $str;
				break;
			case 'collection':
			case 'collectiontype':
				$result = '1'; // TODO: Properties 'collection' und 'collectiontype' untersttzen
				break;
		}

		if ( empty( $result ) )
		{
			return FALSE;
		}

		return ' ' . $result . ' ';
	}

	
	function tag2column( $tag )
	{
		// returns internal db column name for Picturemaxx tag name
		// $tag can be a single tag or a list of tags

		if ( empty( $tag ) )
		{
			return FALSE;
		}

		if ( ! is_array( $tag ) )
		{
			return isset( $this->metadata[ $tag ] ) ? $this->metadata[ $tag ] : FALSE;
		}
		
		foreach ( $tag as $item )
		{
			if ( isset( $this->metadata[ $item ] ) )
			{
				$result[] = $this->metadata[ $item ];
			}
		}

		return $result;
	}


	function column2tag( $column )
	{
		// returns Picturemaxx tag name for internal db column name

		$trans = array_flip( $this->metadata );
		return isset( $trans[ $column ] ) ? $trans[ $column ] : FALSE;
	}

	
	function load_globals()
	{
		if ( ! empty( $this->globals ) )
		{
			return TRUE;
		}

		$sql = "SELECT `data` FROM {$this->tblRoot} WHERE `vtid` = 1";
		$result = $this->db->query( $sql );

		if ( ! $result || ! $result->num_rows )
		{
			return FALSE;
		}

		$row = $result->fetch_assoc();
		$data = unserialize( $row['data'] );
		$this->globals = $data['globals'];
		return TRUE;
	}


	function get_permission( $ref, $resolution, $type = 'download' )
	{
		if ( ! $this->load_globals() )
		{
			return FALSE;
		}

		if ( ! in_array( $type, array( 'view', 'download' ) ) )
		{
			return FALSE;
		}
		
		if ( $resolution == 'thumb' )
		{
			return TRUE;
		}

		if ( $resolution == 'preview' )
		{
			if ( $type == 'view' )
			{
				return TRUE;
			}

			switch ( $this->globals['pxtcPrivilegeLoresDownload'] )
			{
				case '1':
				case '2':
					return TRUE;

				case '3':
					$loresGroups = explode( ',', $this->globals['pxtcGroupsLoresDownload'] );
					return array_intersect( $loresGroups, $this->get_user_groups('name') );
			}
			
			return FALSE;
		}

		if ( $resolution == 'highres' )
		{
			if ( $type == 'view' )
			{
				return FALSE;
			}

			switch ( $this->globals['pxtcPrivilegeHiresDownload'] )
			{
				case '1':
				case '2':
					return TRUE;

				case '3':
					$hiresGroups = explode( ',', $this->globals['pxtcGroupsHiresDownload'] );
					return array_intersect( $hiresGroups, $this->get_user_groups('name') );
			}

			$sql = "SELECT c.`accessRights`, d.`downloadallowed` 
							FROM `{$this->tblContent}` c, `{$this->tblImagelists}` d 
							WHERE c.`containerID` = d.`vtid` AND d.`dateiname` LIKE '$ref%'";
			$result = $this->db->query( $sql );

			if ( ! $result || ! $result->num_rows )
			{
				return FALSE;
			}
			
			$row = $result->fetch_assoc();
			return substr( $row['accessRights'], -1 ) < 4 && $row['downloadallowed'] == '1';
		}
		
		return FALSE;
	}

	private function read_config( $file )
	{
		$config = file( $file );
		array_shift( $config );
		array_pop( $config );
		$result = array();

		foreach ( $config as $line )
		{
			$line = trim( $line );

			if ( ! empty( $line ) && $line[0] != ';' && strpos( $line, '=' ) )
			{
				list( $field, $value ) = explode( '=', $line, 2 );
				$result[ trim( $field ) ] = trim( trim( $value ), '"\'' );
			}
		}

		return $result;
	}
}

// end of file PmConnector.class.php
