<?php

class Cleverreach {
	
	var $method; // method used for exchanging data with the Cleverreach server ('socket' or 'curl')
	
	function __construct( $account_id, $api_key, $method = 'socket' )
	{
		$this->api_key = $api_key;
		$this->method = $method;

		$this->server =  $account_id . '.cleverreach.de';
		$this->path =  'soap/interface_v5.0.php?wsdl';

		if ( function_exists( 'date_default_timezone_set' ) )
		{
			date_default_timezone_set( @date_default_timezone_get() );
		}

	}
	
	function addReceiver( $list_id, $data )
	{
		if ( is_string( $data ) )
		{
			$data = array( 'email' => $data );
		}
		elseif ( empty( $data['email'] ) )
		{
			return 'ERROR: Invalid receiver data';
		}

		$body = '<ns1:receiverAdd>';
		$body .= "<apiKey xsi:type=\"xsd:string\">{$this->api_key}</apiKey>";
		$body .= "<listId xsi:type=\"xsd:int\">{$list_id}</listId>";
		$body .= '<subscriberData xsi:type="ns1:receiverData">' . $this->_arr2xml( $data ) . '</subscriberData>';
		$body .= '</ns1:receiverAdd>';
		$response = $this->_request( $body, 'receiverAdd' );

		if ( ! $response )
		{
			return "ERROR: Request via {$this->method} failed";
		}

		$result = $this->xml_extract( $response, 'id', 'single' );

		if ( ! $result )
		{
			return $response;
		}

		return $result;
	}
	
	function getLists()
	{
		$body = "<ns1:groupGetList><apiKey xsi:type=\"xsd:string\">{$this->api_key}</apiKey></ns1:groupGetList>";
		$response = $this->_request( $body, 'groupGetList' );

		if ( $response === FALSE )
		{
			return FALSE;
		}

		$lists = $this->xml_extract( $response, 'id|name', 'records' );
		return $lists;
	}

	function _arr2xml( $array )
	{
		$result = '';
	
		foreach ( $array as $name => $value )
		{
			if ( is_array( $value ) )
			{
				$result .= "<{$name}>" . $this->_arr2xml( $value ) . "</{$name}>";
			}
			else
			{
				$result .= "<{$name}>" . htmlspecialchars( $value, ENT_COMPAT | ENT_HTML401, 'ISO-8859-1' ) . "</{$name}>";
			}
		}
	
		return $result;
	}

	function _request( $body, $action = '' )
	{
		$request = <<< EOT
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="CRS" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
	<SOAP-ENV:Body>{$body}</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
EOT;
		// die( $request );

		switch ( $this->method )
		{
			case 'socket':
				return $this->_send_via_socket( $request, $action );
			case 'curl':
				return $this->_send_via_curl( $request, $action );
			default:
				return FALSE;
		}
	}

	function _send_via_socket( $request, $action = '' )
	{
		$headers = array();
		if ( ! empty( $action ) )
		{
			$headers[] = "SOAPAction: {$this->server}/{$this->path}/{$action}";
		}
		require_once dirname( __FILE__ ) . '/tcpconnection.class.php';
		require_once dirname( __FILE__ ) . '/httpconnection.class.php';
		$this->http = new httpconnection( $this->server, 80 );
		$this->response = $this->http->post( $this->path, $request, array(), array(), array(), '', '', $headers );

		if ( $this->response['head']['code'] != 200 )
		{
			return FALSE;
		}

		return $this->response['body'];
	}

	function _send_via_curl( $request, $action = '' )
	{
		$url = "http://{$this->server}/{$this->path}";
		$headers[] = 'Content-Type: text/xml; charset=utf-8';

		if ( ! empty( $action ) )
		{
			$headers[] = "SOAPAction: {$url}/{$action}";
		}

		$curl = curl_init();
		curl_setopt( $curl, CURLOPT_URL, $url );
		curl_setopt( $curl, CURLOPT_FORBID_REUSE, TRUE );
		curl_setopt( $curl, CURLOPT_FRESH_CONNECT, TRUE );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, TRUE );
		curl_setopt( $curl, CURLOPT_SSL_VERIFYPEER, FALSE );
		curl_setopt( $curl, CURLOPT_POST, TRUE );
		curl_setopt( $curl, CURLOPT_POSTFIELDS, $request );
		curl_setopt( $curl, CURLINFO_HEADER_OUT, TRUE );
		curl_setopt( $curl, CURLOPT_HTTPHEADER, $headers );

		$this->response = curl_exec( $curl );
		$info = curl_getinfo( $curl );
		curl_close( $curl );
		
		if ( $info['http_code'] != 200 )
		{
			return FALSE;
		}

		return $this->response;
	}

	function xml_extract( $xml, $element_match, $type = 'single' )
	{
		$matches = array();
		switch ( $type )
		{
			case 'single':
				$regex = '!<(' . $element_match . ')[^>]*>([\s\S]*?)</\1>!';
				if ( ! preg_match( $regex, $xml, $matches ) ) return FALSE;
				return $matches[2];
				break;
			case 'all':
				$regex = '!<(' . $element_match . ')[^>]*>([\s\S]*?)</\1>!';
				if ( ! preg_match_all( $regex, $xml, $matches, PREG_SET_ORDER ) ) return FALSE;
				foreach ( $matches as $item ) $result[] = array( $item[1] => $item[2] );
				return $result;
				break;
			case 'records':
				$regex = '!<(' . $element_match . ')[^>]*>([^<]*)</\1>!';
				if ( ! preg_match_all( $regex, $xml, $matches, PREG_SET_ORDER ) ) return FALSE;
				$result = array();
				$record = array();
				$first_element = '';
				foreach ( $matches as $item )
				{
					if ( empty( $first_element ) ) $first_element = $item[1];
					elseif ( $item[1] == $first_element ){$result[] = $record; $record = array();}
					$record[ $item[1] ] = $item[2];
				}
				$result[] = $record;
				return $result;
				break;
			default:
				return FALSE;
		}
	}

}

// end of file cleverreach.class.php
