<?php

class Easybill {
	
	public $http; // Simple_HTTP object
	
	public function __construct( $auth_key, $method = 'socket' )
	{
		if ( function_exists( 'date_default_timezone_set' ) )
		{
			date_default_timezone_set( @date_default_timezone_get() );
		}

		$this->auth_key = $auth_key;
		require_once dirname( __FILE__ ) . '/simple_http.class.php';
		$this->http = new Simple_HTTP( 'api.easybill.de', TRUE );
		$this->http->useCurl = $method == 'curl';
	}
	

	public function get_customer( $customer_id )
	{
		$path = "/customers/{$customer_id}";
		$response = $this->_request( $path );

		if ( ! isset( $response['status'] ) || $response['status'] != 200 )
		{
			return FALSE;
		}

		$data = json_decode( $response['body'], TRUE );
		return $this->_convert_names( $data, 'customer_from_rest' );
	}
	

	public function get_customer_by_number( $customer_number )
	{
		$path = "/customers?number={$customer_number}";
		$response = $this->_request( $path );

		if ( ! $response )
		{
			return FALSE;
		}

		$data = json_decode( $response['body'], TRUE );
		return empty( $data['items'][0] ) ? FALSE : $this->_convert_names( $data['items'][0], 'customer_from_rest' );
	}
	

	public function search_customers( $searchstring )
	{
		$searchstring = urlencode( utf8_encode( $searchstring ) );
		$path = "/customers?last_name={$searchstring}";
		$response = $this->_request( $path );

		if ( ! $response )
		{
			return FALSE;
		}

		$data = json_decode( $response['body'], TRUE );
		
		if ( empty( $data['items'][0] ) )
		{
			return FALSE;
		}
		
		foreach ( $data['items'] as $item )
		{
			$items[] = $this->_convert_names( $item, 'customer_from_rest' );
		}
		
		return $items;
	}
	

	public function set_customer( $customer )
	{
		$customer = $this->_convert_names( $customer, 'customer_to_rest' );
		
		if ( empty( $customer['id'] ) )
		{
			// create new record
			$path = "/customers";
			$response = $this->_request( $path, json_encode( $customer ) );
		}
		else
		{
			// update existing record
			$path = "/customers/{$customer['id']}";
			unset( $customer['id'] );
			$response = $this->_request( $path, json_encode( $customer ), TRUE );
		}

		if ( ! $response )
		{
			return FALSE;
		}

		$data = json_decode( $response['body'], TRUE );
		return $this->_convert_names( $data, 'customer_from_rest' );
	}
	

	public function create_document( $document )
	{
		$sendImmediately = ! empty( $document['sendasemail'] );
		unset( $document['sendasemail'] );
		$document = $this->_convert_names( $document, 'document_to_rest' );
		$response = $this->_request( '/documents', json_encode( $document ) );

		if ( ! $response )
		{
			return FALSE;
		}
		
		$data = json_decode( $response['body'], TRUE );
		
		if ( empty( $data['id'] ) )
		{
			return FALSE;
		}

		$response = $this->_request( "/documents/{$data['id']}/done", '', TRUE );
		$data = json_decode( $response['body'], TRUE );

		if ( $sendImmediately )
		{
			$this->_request( "/documents/{$data['id']}/send/email", '{}' );
		}
		
		return $this->_convert_names( $data, 'document_from_rest' );
	}
	

	private function _request( $path, $postData = NULL, $usePutMethod = FALSE )
	{
		$path = 'rest/v1/' . ltrim( $path, '/' );
		$method = is_null( $postData ) ? 'GET' : ( $usePutMethod ? 'PUT' : 'POST' );
		$content = (string) $postData;
		$contentType = 'application/json';
		$headers = array( "Authorization: Bearer {$this->auth_key}" );
		$response = $this->http->send( $method, $path, $content, $contentType, $headers );
		return $response;
	}


	private function _convert_names( $array, $rule )
	{
		$rules['customer_from_rest'] = array (
			'id' => 'customerID',
			'number' => 'customerNumber',
			'company_name' => 'companyName',
			'first_name' => 'firstName',
			'last_name' => 'lastName',
			'vat_identifier' => 'ustid',
			'zip_code' => 'zipCode'
		);
			
		$rules['customer_to_rest'] = array (
			'customerID' => 'id',
			'customerNumber' => 'number',
			'companyName' => 'company_name',
			'firstName' => 'first_name',
			'lastName' => 'last_name',
			'ustid' => 'vat_identifier',
			'zipCode' => 'zip_code'
		);
		
		$rules['item_from_rest'] = array (
			'type' => 'positionType',
			'number' => 'itemNumber',
			'description' => 'itemDescription',
			'quantity' => 'count',
			'single_price_net' => 'singlePriceNetto',
			'vat_percent' => 'ustPercent'
		);
			
		$rules['item_to_rest'] = array (
			'positionType' => 'type',
			'itemNumber' => 'number',
			'itemDescription' => 'description',
			'count' => 'quantity',
			'singlePriceNetto' => 'single_price_net',
			'ustPercent' => 'vat_percent'
		);
		
		$rules['document_from_rest'] = array (
			'id' => 'documentID',
			'number' => 'documentNumber',
			'customer_id' => 'customerID',
			'customer_number' => 'customerNumber',
			'type' => 'documentType',
			'contact_label' => 'contactLabel',
			'contact_text' => 'contactText',
			'pdf_template' => 'templateName',
			'vat_option' => 'taxOptions'
		);
			
		$rules['document_to_rest'] = array (
			'customerID' => 'customer_id',
			'documentType' => 'type',
			'contactLabel' => 'contact_label',
			'contactText' => 'contact_text',
			'templateName' => 'pdf_template',
			'taxOptions' => 'vat_option'
		);
		
		if ( empty( $rules[ $rule ] ) )
		{
			return $array;
		}
			
		$newArray = array ();

		foreach ( $array as $key => $value )
		{
			if ( $rule == 'customer_from_rest' && $key == 'emails' )
			{
				$newArray['email'] = $value[0];
				continue;
			}

			if ( $rule == 'customer_from_rest' && $key == 'salutation' )
			{
				$newArray['salutation'] = $value == 2 ? 'Frau' : ( $value == 3 ? 'Firma' : 'Herr' );
				continue;
			}

			if ( $rule == 'customer_to_rest' && $key == 'email' )
			{
				$newArray['emails'] = array ( $value );
				continue;
			}

			if ( $rule == 'customer_to_rest' && $key == 'salutation' )
			{
				$newArray['salutation'] = $value == 'Frau' ? 2 : ( $value == 'Firma' ? 3 : 1 );
				continue;
			}

			if ( $rule == 'document_from_rest' && $key == 'items' )
			{
				$newArray['documentPosition'] = array_map( array ( $this, '_convert_names' ), $value, array_fill( 0, count( $value ) ,'item_from_rest' ) );
				continue;
			}

			if ( $rule == 'document_to_rest' && $key == 'documentPosition' )
			{
				$newArray['items'] = array_map( array ( $this, '_convert_names' ), $value, array_fill( 0, count( $value ) ,'item_to_rest' ) );
				continue;
			}

			$newKey = isset( $rules[ $rule ][ $key ] ) ? $rules[ $rule ][ $key ] : $key;
			$newArray[ $newKey ] = $value;
		}
		
		return $newArray;
	}
}

// end of file easybill-rest.class.php
