<?php

if ( class_exists('mysqli') )
{
	class mysqli_extended extends mysqli {
		
		var $tablePrefix = '';
		var $logFunction;
		var $debug = FALSE;
		
		
		function __construct( $server, $username, $password, $database, $port = NULL, $socket = NULL )
		{
			parent::__construct( $server, $username, $password, $database, $port, $socket );
		}


		public function query( $sql, $resultmode = NULL )
		{
			if ( $this->tablePrefix )
			{
				$sql = str_replace( '`%', '`' . $this->tablePrefix, $sql );
			}

			$startTime = microtime( TRUE );
			$result = parent::query( $sql, $resultmode );
			$elapsedTime = microtime( TRUE ) - $startTime;

			if ( $result && ! $this->debug )
			{
				return $result;
			}
			
			$trace = debug_backtrace();
			array_shift( $trace );
	
			foreach ( $trace as $call )
			{
				$chain[] = str_replace( 'ƒ', '', $call['function'] );
			}

			$this->_log( '---------------------------------------------------------------------', ! $result );
			$this->_log( 'QUERY by ' . implode( ' < ', $chain ), ! $result );
			$this->_log( $sql, ! $result );
			$message = "RESULT [{$elapsedTime} ms]:";

			if ( $this->errno > 0 )
			{
				$this->_log( "{$message} MySQL-Error #{$this->errno}: {$this->error}", TRUE );
			}
			elseif ( is_object( $result ) )
			{
				$this->_log( "{$message} {$result->num_rows} record(s) found" );
			}
			else
			{
				$this->_log( "{$message} {$this->affected_rows} record(s) affected" );
			}

			$this->_log( '---------------------------------------------------------------------', ! $result );
			return $result;
		}
		
		
		private function _log( $message, $isError = FALSE )
		{
			if ( ! is_callable( $this->logFunction ) )
			{
				return;
			}

			call_user_func( $this->logFunction, $message, $isError );
		}
	}
}



class mysqli_php4 {
	
	var $connection;

	var $affected_rows;
	var $connect_errno;
	var $connect_error;
	var $errno;
	var $error;
	var $insert_id;
	var $server_info;
	

	function __construct( $server, $username, $password, $database, $port = NULL, $socket = NULL )
	{
		if ( empty( $socket ) )
		{
			$result = mysql_connect( empty( $port ) ? $server : $server . ':' . $port, $username, $password );
		}
		else
		{
			$result = mysql_connect( $socket, $username, $password );
		}
		
		if ( ! $result )
		{
			$this->connect_errno = mysql_errno();
			$this->connect_error = mysql_error();
			return FALSE;
		}
		
		$this->connection = $result;
		$this->server_info = mysql_get_server_info( $result );

		if ( ! empty( $database ) )
		{
			return $this->select_db( $database );
		}

		return TRUE;
	}

	
	function close()
	{
		if ( is_resource( $this->connection ) )
		{
			mysql_close( $this->connection );
		}
	}


	function select_db( $database )
	{
		return mysql_select_db( $database );
	}


	function insert_id()
	{
		return mysql_insert_id( $this->connection );
	}


	function query( $query )
	{
		$this->errno = NULL;
		$this->error = NULL;
		$this->affected_rows = NULL;
		$this->insert_id = NULL;
		$result = mysql_query( $query, $this->connection );
		
		if ( ! $result )
		{
			$this->errno = mysql_errno( $this->connection );
			$this->error = mysql_error( $this->connection );
			return FALSE;
		}

		if ( ! is_resource( $result ) )
		{
			$this->affected_rows = mysql_affected_rows( $this->connection );
			$this->insert_id = mysql_insert_id( $this->connection );
			return TRUE;
		}

		return new mysqli_result_php4( $result );
	}

	
	function real_escape_string( $string )
	{
		if ( ! function_exists( 'mysql_real_escape_string' ) )
		{
			return mysql_escape_string( $string ); // for PHP < 4.3
		}

		return mysql_real_escape_string( $string, $this->connection );
	}


	function escape_string( $string )
	{
		return $this->real_escape_string( $string );
	}


	function set_charset()
	{
		return TRUE;
	}
	
}


class mysqli_result_php4 {
	
	var $resultset;
	var $num_rows;
	

	function __construct( $resultset )
	{
		$this->resultset = $resultset;
		$this->num_rows = mysql_num_rows( $resultset );
	}

	
	function data_seek( $row_number )
	{
		return mysql_data_seek( $this->resultset, $row_number );
	}
	

	function fetch_assoc()
	{
		return mysql_fetch_assoc( $this->resultset );
	}


	function fetch_row()
	{
		return mysql_fetch_row( $this->resultset );
	}
	

	function free_result()
	{
		return mysql_free_result( $this->resultset );
	}
	
}
